#ifndef CANNYEDGEDETECTOR_H
#define CANNYEDGEDETECTOR_H
#include <stdlib.h>
#include <stdio.h>
#include "wx/wx.h"
#include <boost/shared_ptr.hpp>
 #include <boost/numeric/ublas/matrix.hpp>

using namespace boost;
using namespace boost::numeric::ublas;

class CannyEdgeDetector {
    private:
        float sigma;
        unsigned char low;
        unsigned char high;
        CannyEdgeDetector() {
        }
        float gauss (float x, float sigma);
        float dGauss (float x, float sigma);
        void seperable_convolution (shared_ptr<wxImage>&img, float *mask, int width, matrix<float>&smx, matrix<float>&smy);
        void dxy_seperable_convolution
        (matrix<float> &imx, matrix<float> &imy, int nr, int nc, float *mask, int width, matrix<float>&smx, matrix<float>&smy);
        float norm (float x, float y);
        void nonmax_suppress (shared_ptr<wxImage>&mag_img, matrix<float>&dx, matrix<float>&dy, int nr, int nc);
        bool range (shared_ptr<wxImage>img, int i, int j);
        int trace (int i, int j, unsigned char low, shared_ptr<wxImage>&img, shared_ptr<wxImage>&mag);
        void hysteresis (shared_ptr<wxImage>&mag, unsigned char high, unsigned char low, shared_ptr<wxImage>&img);
    public:
        void run(shared_ptr<wxImage>&img);
        void setParameters(float s, unsigned char l, unsigned char h) {
            sigma = s;
            low = l;
            high = h;
        }
       static CannyEdgeDetector* getInstance() {
          static CannyEdgeDetector *instance = 0;
          if (!instance)
            instance = new CannyEdgeDetector();
          return instance;
       }

};



#endif
