#include "EZGimpySolver.h"
#include <boost/graph/graph_traits.hpp>
#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/dijkstra_shortest_paths.hpp>
#include "Munkres.h"
#include "Utility.h"
#include "TPS.h"
#include "GeneralizedShapeContext.h"

typedef tuple<int,int,int> trigma;
typedef adjacency_list<vecS, vecS, bidirectionalS> Graph;


//****************************************************************************/
//* Define as imagens de treinamento
//****************************************************************************/
void EZGimpySolver::SetTrainingImages(std::vector<Image*> &trainingImages) {
    this->trainingImages = trainingImages;
}

//****************************************************************************/
//* Define a imagem do captcha
//****************************************************************************/
void EZGimpySolver::SetImage(Image *img) {
    this->img = img;
}

//****************************************************************************/
//* Devolve as tuplas com hipóteses de localizações de letras
//* Todos os pontos associados à uma determinada letra são adicionados
//* ao vetor LETTERS
//****************************************************************************/
std::vector<DistanceAndLetter> EZGimpySolver::getTuples(std::vector<Point> LETTERS[26], char c) {
    std::vector<SCPairLetter> scs;
    std::vector<DistanceAndLetter> distances;
    wxMemoryDC dc;
    double sum;
    double cost;

    if (img == NULL)
        return distances;

    std::vector<Point> allpts = img->getAllPoints();
    int numSC = 0.3 * allpts.size();

    std::vector<Point> pts1 = img->getPoints(numSC);

    std::vector<ShapeContext*> SCQuery[26];
    std::vector<ShapeContext*> SCShape[26];
    int med[26];

    for (int n = 0; n < 26; n++) {
        std::vector<Point> points = trainingImages[n]->getAllPoints();
        // Calculo da distancia media
        int m = median(points);
        med[n] = m;
        // Calculo dos shape contexts
        for (unsigned int i = 0; i < pts1.size(); i++) {
            ShapeContext *sc = new GeneralizedShapeContext(pts1[i]);
            sc->compute(allpts,med[n]);
            SCQuery[n].push_back(sc);
        }
        trainingImages[n]->computeShapeContexts(SAMPLE_SIZE,med[n]);
        SCShape[n] = trainingImages[n]->getShapeContexts();
    }

    // Para cada letra
    for (int n = 0; n < 26; n++) {
        std::vector<CostAndIndices> costs;
        //Para cada SC da letra
        for (unsigned int j = 0; j < SCShape[n].size(); j++) {
            ShapeContext *scShape = SCShape[n][j];
            // Para cada SC da imagem de consulta
            for (int i = 0; i < numSC; i++) {
                ShapeContext *scQuery = SCQuery[n][i];
                cost = scQuery->distance(scShape);
                costs.push_back(make_tuple(cost,scQuery,scShape));
            }

            // Temos os pares com os melhores custos
            sort(costs.begin(), costs.end(), compareCost);
            for (unsigned int i = 0; i < 0.1*costs.size(); i++) {
                    scs.push_back(make_tuple(costs[i].get<0>(),costs[i].get<1>(),costs[i].get<2>(),n));
            }
            costs.clear();
        }
    }

    sort(scs.begin(), scs.end(), comparePair);

    // Esquema de votacao
    int votes[numSC][26];
    for (int t = 0; t < numSC; t++)
        for (unsigned int q = 0; q < 26; q++)
            votes[t][q] = 0;

    for (unsigned int i = 0; i < 0.7*scs.size(); i++) {
        int letra = scs[i].get<3>();
        int x = scs[i].get<1>()->getPoint().x;
        int y = scs[i].get<1>()->getPoint().y;
        for (int t = 0; t < numSC; t++) {
            Point pt = pts1[t];
            if (pt.x == x && pt.y == y)
                votes[t][letra]++;
        }
    }

    for (int t = 0; t < numSC; t++) {
        int max = 0;
        int max_ind = 0;
        for (unsigned int q = 0; q < 26; q++) {
            if (votes[t][q] > max) {
                max = votes[t][q];
                max_ind = q;
            }
        }

        if (max > 0) {
            for (unsigned int q = 0; q < 26; q++)
                if (votes[t][q] == max)
                    LETTERS[q].push_back(pts1[t]);
        }
    }

    for (int n = 0; n < 26; n++) {
        int aux[LETTERS[n].size()];
        for (unsigned int i = 0; i < LETTERS[n].size(); i++) {
            aux[i] = 0;
        }
        // Agrupamento de pontos por proximidade
        std::vector<Point>  groups[50];
        int num = 0;
        for (unsigned int i = 0; i < LETTERS[n].size(); i++) {
            if (aux[i] == 0) {
                aux[i] = 1;
                groups[num].push_back(LETTERS[n][i]);
                Point pt1 = LETTERS[n][i];

                for (unsigned int j = 0; j < LETTERS[n].size(); j++) {
                    if (aux[j] == 0) {
                        Point pt2 = LETTERS[n][j];
                        double distx = abs(pt1.x - pt2.x);
                        if (distx <= 20) {
                            groups[num].push_back(LETTERS[n][j]);
                            aux[j] = 1;
                        }
                    }
                }
                num++;
            }
        }

        /*************************************************/
        /*  Cálculo da distância                        */
        /*************************************************/
        double N = 1;
        for (int l = 0; l < 50 && l < num; l++) {
            double d = 0;
            if (groups[l].size() >= 1) {
                for (unsigned int i = 0; i < groups[l].size(); i++) {
                    Point ptQuery = groups[l][i];
                    ShapeContext *scQuery = new GeneralizedShapeContext(ptQuery);
                    scQuery->compute(allpts,med[n]);

                    double sum = 0;
                    double value = 0;
                    for (int j = 0; j < 26; j++) {
                        double minValue = scQuery->distance(SCShape[n][0]);
                        for (unsigned int k = 1; k < SCShape[j].size(); k++) {
                            double distance = scQuery->distance(SCShape[j][k]);
                            if (distance < minValue) {
                                minValue = distance;
                            }
                        }

                        if (j == n)
                            value = minValue;

                        sum = sum + minValue;
                    }

                    N = sum/26.0;

                    if (N > 0)
                        d = d + value/N;
                }

                d = d/groups[l].size();
                char c = 'a' + n;

                /*************************************************/
                /* Estimativa da posição da letra                */
                /*************************************************/
                sum = 0;
                Point position;
                double x = 0;
                double y = 0;
                for (unsigned int u = 0;  u < groups[l].size(); u++) {
                    Point ptQuery = groups[l][u];
                    ShapeContext *scQuery = new GeneralizedShapeContext(ptQuery);
                    scQuery->compute(allpts, med[n]);

                    double minValue = scQuery->distance(SCShape[n][0]);
                    for (unsigned int k = 1; k < SCShape[n].size(); k++) {
                        double distance = scQuery->distance(SCShape[n][k]);
                        if (distance < minValue) {
                            minValue = distance;
                        }
                    }

                    Point pt1 = scQuery->getPoint();
                    double weight = 1-minValue;

                    x += weight*(pt1.x);
                    y += weight*(pt1.y);
                }

                double M = 0;
                for (unsigned int u = 0; u < groups[l].size(); u++) {
                    Point ptQuery = groups[l][u];
                    ShapeContext *scQuery = new GeneralizedShapeContext(ptQuery);
                    scQuery->compute(allpts, med[n]);
                    double minValue = scQuery->distance(SCShape[n][0]);
                    for (unsigned int k = 1; k < SCShape[n].size(); k++) {
                        double distance = scQuery->distance(SCShape[n][k]);
                        if (distance < minValue) {
                            minValue = distance;
                        }
                    }
                    M += (1-minValue);
                }
                x /= M;
                y /= M;

                position.x = (int) x;
                position.y = (int) y;

                std::cout <<  "Distancia = " << d << " Letra: " << c << " Posicao: (" << position.x << "," << position.y << ")" << std::endl;
                distances.push_back(make_tuple(d, c, position));
            }
        }
    }

    // Ordena as distâncias obtidas
    sort(distances.begin(), distances.end(), compareDistanceAndLetter);

    // Mantem as 100 primeiras
    if (distances.size() > 100)
        distances.erase(distances.begin() + 100, distances.end());

    return distances;
}

//****************************************************************************/
//* Recebe as tuplas de hipóteses e o vetor com as palavras do dicionario
//* Devolve um vetor com todas as palavras possíveis
//****************************************************************************/
std::vector<std::string> EZGimpySolver::getWordslist(std::vector<DistanceAndLetter> distances, std::vector<std::string> words) {
    matrix<double> Cost (SAMPLE_SIZE,SAMPLE_SIZE);
    std::vector< tuple<double,std::string> > averages;

    std::cout << "Total de hipoteses: " << distances.size() <<"\n";
    int num = distances.size();
    int num_vertices = distances.size();

    double lettersCost[26];
    for (int l = 0; l < 26; l++)
        lettersCost[l] = -1;

    int num_words = words.size();

    //*********************************************************/
    //* Contrução do grafo
    //*********************************************************/
    Graph graph(num_vertices);
    int pos1, pos2;
    for (int i = 0; i < num; ++i) {
        pos1 = distances[i].get<2>().x;
        for (int j = 0; j < num; ++j) {
            pos2 = distances[j].get<2>().x;
            if ((pos2 - pos1 >= 10) && (pos2 - pos1 <= 35))
                add_edge(i, j, graph);
        }
    }

    typedef property_map<Graph, vertex_index_t>::type IndexMap;
    IndexMap index = get(vertex_index, graph);

    //*****************************************************************************/
    //* Obtem todos os trigmas (sequencia de tres letras) presentes no grafo
    //*****************************************************************************/
	 std::vector<trigma> trigmas;
    char t1,t2,t3,p1,p2,p3;

    typedef graph_traits<Graph>::vertex_iterator vertex_iter;
    std::pair<vertex_iter, vertex_iter> vp;
    for (vp = vertices(graph); vp.first != vp.second; ++vp.first) {
		t1 = distances[*vp.first].get<1>();
		graph_traits<Graph>::adjacency_iterator ai, aend;
        for (tie(ai, aend) = adjacent_vertices(*vp.first,graph); ai != aend; ++ai) {
            t2 =  distances[*ai].get<1>();
			graph_traits<Graph>::adjacency_iterator ai2, aend2;
			for (tie(ai2, aend2) = adjacent_vertices(*ai,graph); ai2 != aend2; ++ai2) {
			    t3 = distances[*ai2].get<1>();
                trigmas.push_back(make_tuple(*vp.first,*ai,*ai2));
			}
		}
    }

    std::cout << "Total de trigmas = " << trigmas.size() << std::endl;

    std::vector<trigma> words_trigmas[num_words];
	for (int i = 0; i < num_words; i++) {
		for (unsigned int j = 0; j < words[i].size()-2; j++) {
			words_trigmas[i].push_back(make_tuple(words[i][j],words[i][j+1],words[i][j+2]));
		}
	}

    //*****************************************************************************/
    //* Obtem as palavras possiveis
    //*****************************************************************************/
    std::vector<std::string> possible;
    int ok;
	for (int k = 0; k < num_words; k++) {
	    std::vector<int> posi[num_words][words_trigmas[k].size()];
		unsigned int total = 0;
		for (unsigned int i = 0; i < words_trigmas[k].size(); i++) {
		    posi[k][i].clear();
			t1 = words_trigmas[k][i].get<0>();
			t2 = words_trigmas[k][i].get<1>();
			t3 = words_trigmas[k][i].get<2>();
			ok = 0;
			for (unsigned int j = 0; j < trigmas.size(); j++) {
				p1 = distances[trigmas[j].get<0>()].get<1>();
				p2 = distances[trigmas[j].get<1>()].get<1>();
				p3 = distances[trigmas[j].get<2>()].get<1>();
				if ((t1 == p1) && (t2 == p2) && (t3 == p3)) {
					ok = 1;
                    posi[k][i].push_back(distances[trigmas[j].get<0>()].get<2>().x);
				}
			}
			if (ok) {
                total++;
			}
		}

		if ((total == words_trigmas[k].size())) {
		    int ok = 1;
		    int rightLimit = 0, right = INT_MAX;
            for (int i = (int) words_trigmas[k].size()-1; i >=0 ; i--) {
                bool isPossible = false;
                for (unsigned int j = 0; j < posi[k][i].size() ; j++) {
                    if (posi[k][i][j] < right) {
                        isPossible = true;
                        if (posi[k][i][j] > rightLimit)
                            rightLimit = posi[k][i][j];
                    }
                }

                right = rightLimit;
                rightLimit = 0;
                if (!isPossible) {
                   ok = 0;
                   break;
                }
            }
            if (ok) {
                possible.push_back(words[k]);
            }
		}
	}

    return possible;
}


//*****************************************************************************/
//* Recebe um vetor com todas as palavras possíveis.
//* Devolve o ranking dessas palavras.
//*****************************************************************************/
std::vector< tuple<double,std::string> > EZGimpySolver::solve(std::vector<std::string> words) {
    double optimalCost;
    matrix<double> Cost (SAMPLE_SIZE,SAMPLE_SIZE);
    std::vector< tuple<double,std::string> > averages;
    Munkres munkresSolver;

    double lettersCost[26];
    for (int l = 0; l < 26; l++)
        lettersCost[l] = -1;

	for (unsigned int k = 0; k < words.size(); k++) {
                double average = 0;
                double cost = 0;
                for (unsigned m = 0; m < words[k].size(); m++) {
                    char c = words[k][m];

                    if (lettersCost[c-'a'] < 0) {
                        int med = median(trainingImages[c-'a']->getAllPoints());
                        trainingImages[c-'a']->computeShapeContexts(60, med);
                        img->computeShapeContexts(250, med);
                        std::vector<ShapeContext*> SC1 = trainingImages[c-'a']->getShapeContexts();
                        std::vector<ShapeContext*> SC2 = img->getShapeContexts();

                        matrix<double> Cost (SC1.size(),SC2.size());
                        for (unsigned int i = 0; i < SC1.size(); i++) {
                            ShapeContext *sc1 = SC1[i];
                            for (unsigned int j = 0; j < SC2.size(); j++) {
                                ShapeContext *sc2 = SC2[j];
                                Cost(i,j) = sc1->distance(sc2);
                            }
                        }

                        int dim = (Cost.size1() >= Cost.size2()) ? Cost.size1():Cost.size2();
                        vector<int> results(dim);

                        munkresSolver.compute(Cost,results);
                        optimalCost = 0;
                        for (unsigned h = 0; h < Cost.size1(); h++)
                            optimalCost += Cost(h,results(h));

                        lettersCost[c-'a'] = optimalCost;
                        cost = lettersCost[c-'a'];
                        std::cout << "Letra " << c << ": " << cost << std::endl;
                    }
                    else {
                        cost = lettersCost[c-'a'];
                    }
                    average = average + cost;
                }
                average /= words[k].size();

                averages.push_back(make_tuple(average,words[k]));
            }

    sort(averages.begin(),averages.end(),compareAverage);
    for (unsigned l = 0; l < averages.size(); l++) {
        std::cout << averages[l].get<0>() << " - "  << averages[l].get<1>() << std::endl;
    }

    return averages;
}
