#include "GeneralizedShapeContext.h"
#include <wx/wx.h>
#include <iostream>

using namespace std;

//*****************************************************************************/
//* Calcula o shape context generalizado
//*****************************************************************************/
void GeneralizedShapeContext::compute(std::vector<Point> points, double median) {
    int i, j, radBin, angBin, angle;
    double dist;

    for (i = 0; i < 5; i++) {
        for (j = 0; j < 12; j++) {
            histogram(i,j).x = 0;
            histogram(i,j).y = 0;
        }
    }
	for (unsigned i = 0; i < points.size(); i++) {
            // Invariancia à escala
            dist = hypot(points[i].x - point.x, points[i].y - point.y)/median;

            angle = (int) atan2((point.y - points[i].y),(points[i].x - point.x)) * 180/PI;
            if (dist != 0) {
                if (angle >= 360)
                    angle -= 360;
                if (angle < 0)
                    angle += 360;
                angBin = angularBin(angle);
                radBin = logRadialBin(dist);
                histogram(radBin,angBin).x += points[i].tg.x;
                histogram(radBin,angBin).y += points[i].tg.y;
            }

    }
}

/*****************************************************************************/
// Devolve vetor carregado pelo bin dado por (i,j)
/*****************************************************************************/
Tangent GeneralizedShapeContext::get(int i, int j) {
    return histogram(i,j);
}

/*****************************************************************************/
/* Devolve a distancia em relação a outro shape context                      */
/*****************************************************************************/
double GeneralizedShapeContext::distance(ShapeContext *sc) {
    double sum = 0.0;
    Tangent aux, g, h;

    GeneralizedShapeContext *gsc =  (GeneralizedShapeContext *)sc;
    for (int k = 0; k < 5; k++) {
        for (int l = 0; l < 12; l++) {
            g = this->get(k,l);
            h = gsc->get(k,l);
            aux.x = g.x-h.x;
            aux.y = g.y-h.y;
            double gnorm = hypot(g.x,g.y);
            double hnorm = hypot(h.x,h.y);
            double auxnorm = hypot(aux.x,aux.y);
            if (gnorm + hnorm > 0)
                sum += (auxnorm*auxnorm/(gnorm+hnorm));
        }
    }
    return sum;

}
