#include "CannyEdgeDetector.h"
#include "Image.h"
#include "Utility.h"
#include "GeneralizedShapeContext.h"

//*****************************************************************************/
//* Contrutor
//*****************************************************************************/
Image::Image(std::string file) {
    Image::file = file;
    original = wxImage(wxString(file.c_str(),wxConvUTF8));
    imgContour = shared_ptr<wxImage>(new wxImage(wxString(file.c_str(),wxConvUTF8)));
    img = new wxImage(wxString(file.c_str(),wxConvUTF8));
    CannyEdgeDetector::getInstance()->run(imgContour);
    this->contour = this->getContourBitmap();

}

//*****************************************************************************/
//* Devolve uma string com o nome do arquivo da imagem
//*****************************************************************************/
std::string Image::getFile() {
    return file;
}

//*****************************************************************************/
//* Devolve um vetor com todos os pontos do contorno da imagem
//*****************************************************************************/
std::vector<Point> Image::getAllPoints() {
    if (allPoints.empty()) {
        for( int i = 0; i < img->GetHeight(); i++ )
            for (int j = 0; j < img->GetWidth(); j++)
                if (imgContour->GetGreen(j,i) == 0) {
                    Point pt;
                    pt.x= j;
                    pt.y = i;
                    wxBitmap bmptg = contour.GetSubBitmap(wxRect(pt.x-1, pt.y-1, 3, 3));
                    pt.tg = tangent(bmptg.ConvertToImage());
                    allPoints.push_back(pt);
                }
    }
    return allPoints;
}

//*****************************************************************************/
//* Devolve uma amostra aleatória de pontos do contorno
//*****************************************************************************/
std::vector<Point> Image::getPoints(unsigned int sampleSize) {
    std::vector<Point> allPoints = getAllPoints();
    std::vector<Point> seq;
    if (sampleSize <= allPoints.size())
        sample(allPoints,sampleSize,seq);
    return seq;
}


//*****************************************************************************/
//* Devolve o bitmap da imagem
//*****************************************************************************/
wxBitmap Image::getBitmap() {
    return wxBitmap(*img);
}

//*****************************************************************************/
//* Devolve um bitmap com os pontos do contorno
//*****************************************************************************/
wxBitmap Image::getContourPointsBitmap(unsigned int sampleSize) {
    std::vector<Point> seq;
    seq = getPoints(sampleSize);

    wxBitmap bmp = wxBitmap(img->GetWidth(),img->GetHeight());
    wxMemoryDC dc;
    dc.SelectObject(bmp);

    dc.SetPen(wxPen(*wxBLACK, 1, wxSOLID));
    dc.SetBrush(wxBrush(*wxBLACK, wxSOLID));
    dc.Clear();
    for (unsigned int i = 0; i < seq.size(); i++) {
        Point pt = seq[i];
        dc.DrawCircle(pt.x, pt.y, 1);
    }

    dc.SelectObject(wxNullBitmap);

    return (bmp);
}

//*****************************************************************************/
//* Devolve o bitmap do contorno
//*****************************************************************************/
wxBitmap Image::getContourBitmap() {
    if (imgContour == NULL)
        return NULL;
    return wxBitmap(*imgContour);
}

//*****************************************************************************/
//* Devolve o vetor com os shape contexts
//*****************************************************************************/
std::vector<ShapeContext*> Image::getShapeContexts() {
    return SC;
}

//*****************************************************************************/
//* Calcula uma amostra aleatoria de n shape contexts da imagem
//*****************************************************************************/
void Image::computeShapeContexts(unsigned int n, double median) {
    SC.clear();
    std::vector<Point> allpts = getAllPoints();

    if (n > allpts.size())
        n = allpts.size();

    std::vector<Point> pts;

    sample(allpts, n, pts);
    for (unsigned int i = 0; i < pts.size(); i++) {
        ShapeContext *sc = new GeneralizedShapeContext(pts[i]);
        sc->compute(allpts, median);
        SC.push_back(sc);
    }
}

//*****************************************************************************/
//* Devolve a altura da imagem em pixels
//*****************************************************************************/
int Image::getHeight() {
    return img->GetHeight();
}

//*****************************************************************************/
//* Devolve a largura da imagem em pixels
//*****************************************************************************/
int Image::getWidth() {
    return img->GetWidth();
}

//*****************************************************************************/
//* Devolve a imagem original
//*****************************************************************************/
wxImage Image::getOriginal() {
    return original;
}
