#include "ImagesDialog.h"
#include "Image.h"
#include <wx/dir.h>
#include <wx/imaglist.h>

//(*InternalHeaders(ImagesDialog)
#include <wx/string.h>
#include <wx/intl.h>
#include <wx/bitmap.h>
#include <wx/image.h>
#include <wx/artprov.h>
//*)

#include "Application.h"

//(*IdInit(ImagesDialog)
const long ImagesDialog::ID_BITMAPBUTTON1 = wxNewId();
const long ImagesDialog::ID_BITMAPBUTTON2 = wxNewId();
const long ImagesDialog::ID_BITMAPBUTTON3 = wxNewId();
const long ImagesDialog::ID_LISTCTRL1 = wxNewId();
const long ImagesDialog::ID_BUTTON1 = wxNewId();
const long ImagesDialog::ID_PANEL1 = wxNewId();
//*)

BEGIN_EVENT_TABLE(ImagesDialog,wxDialog)
	//(*EventTable(ImagesDialog)
	//*)
END_EVENT_TABLE()


std::vector<Image*> ImagesDialog::images;
wxImageList* ImagesDialog::imageList = new wxImageList(200, 200);

/*****************************************************************************/
/* Construtor da janela
/*****************************************************************************/
ImagesDialog::ImagesDialog(wxWindow* parent,wxWindowID id,const wxPoint& pos,const wxSize& size) {
	//(*Initialize(ImagesDialog)
	wxBoxSizer* BoxSizer2;
	wxBoxSizer* BoxSizer1;

	Create(parent, wxID_ANY, _("Image List"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE, _T("wxID_ANY"));
	Panel1 = new wxPanel(this, ID_PANEL1, wxPoint(160,192), wxDefaultSize, wxTAB_TRAVERSAL, _T("ID_PANEL1"));
	BoxSizer1 = new wxBoxSizer(wxVERTICAL);
	BoxSizer2 = new wxBoxSizer(wxHORIZONTAL);
	btnAddFile = new wxBitmapButton(Panel1, ID_BITMAPBUTTON1, wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_ADD_BOOKMARK")),wxART_BUTTON), wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW, wxDefaultValidator, _T("ID_BITMAPBUTTON1"));
	btnAddFile->SetDefault();
	BoxSizer2->Add(btnAddFile, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	btnAddDir = new wxBitmapButton(Panel1, ID_BITMAPBUTTON2, wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_FILE_OPEN")),wxART_BUTTON), wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW, wxDefaultValidator, _T("ID_BITMAPBUTTON2"));
	btnAddDir->SetDefault();
	BoxSizer2->Add(btnAddDir, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	BitmapButton1 = new wxBitmapButton(Panel1, ID_BITMAPBUTTON3, wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("wxART_CROSS_MARK")),wxART_BUTTON), wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW, wxDefaultValidator, _T("ID_BITMAPBUTTON3"));
	BitmapButton1->SetDefault();
	BoxSizer2->Add(BitmapButton1, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	BoxSizer1->Add(BoxSizer2, 0, wxALL|wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL, 5);
	ListCtrl = new wxListCtrl(Panel1, ID_LISTCTRL1, wxDefaultPosition, wxSize(200,200), wxLC_LIST|wxLC_AUTOARRANGE|wxLC_SINGLE_SEL|wxSUNKEN_BORDER|wxHSCROLL, wxDefaultValidator, _T("ID_LISTCTRL1"));
	ListCtrl->SetBackgroundColour(wxColour(227,227,200));
	BoxSizer1->Add(ListCtrl, 1, wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	ButtonOK = new wxButton(Panel1, ID_BUTTON1, _("OK"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON1"));
	BoxSizer1->Add(ButtonOK, 0, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
	Panel1->SetSizer(BoxSizer1);
	BoxSizer1->Fit(Panel1);
	BoxSizer1->SetSizeHints(Panel1);
	dirDialog = new wxDirDialog(this, _("Select directory"), wxEmptyString, wxDD_DEFAULT_STYLE, wxDefaultPosition, wxDefaultSize, _T("wxDirDialog"));
	fileDialog = new wxFileDialog(this, _("Select file"), wxEmptyString, wxEmptyString, wxFileSelectorDefaultWildcardStr, wxFD_DEFAULT_STYLE, wxDefaultPosition, wxDefaultSize, _T("wxFileDialog"));

	Connect(ID_BITMAPBUTTON1,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&ImagesDialog::OnbtnAddFileClick);
	Connect(ID_BITMAPBUTTON2,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&ImagesDialog::OnbtnAddDirClick);
	Connect(ID_BITMAPBUTTON3,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&ImagesDialog::OnbtnClearClick);
	Connect(ID_BUTTON1,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&ImagesDialog::OnButtonOKClick);
	//*)

    app = (Application *) wxTheApp;

	ListCtrl->SetImageList(imageList, wxIMAGE_LIST_SMALL);
    for (int i = 0; i < imageList->GetImageCount(); i++) {
        ListCtrl->InsertItem(i,i);
    }

}

/*****************************************************************************/
/* Destrutor
/*****************************************************************************/
ImagesDialog::~ImagesDialog() {
	this->Destroy();
}

/*****************************************************************************/
/* Define o conjunto de imagens selecionadas e fecha a janela.
/*****************************************************************************/
void ImagesDialog::OnButtonOKClick(wxCommandEvent& event) {
    app->setImages(images);
    this->Destroy();
}


/*****************************************************************************/
/* Adicao de uma imagem ao conjunto
/*****************************************************************************/
void ImagesDialog::OnbtnAddFileClick(wxCommandEvent& event) {
    if (fileDialog->ShowModal() == wxID_OK) {
        std::string file(fileDialog->GetPath().mb_str(wxConvUTF8));
        Image *img = new Image(file);
        images.push_back(img);
        int index = imageList->Add(wxBitmap(img->getOriginal()));
        ListCtrl->InsertItem(index,index);
    }
}

/*****************************************************************************/
/* Adicao de todas as imagens de um diretorio ao conjunto
/*****************************************************************************/
void ImagesDialog::OnbtnAddDirClick(wxCommandEvent& event) {
    if (dirDialog->ShowModal() == wxID_OK) {
        wxString strDir = dirDialog->GetPath();

        wxDir dir(strDir);

        if (!dir.IsOpened()) return;

        wxString filename;

        bool cont = dir.GetFirst(&filename,wxT("*.png"));
        while (cont) {
            cont = dir.GetNext(&filename);
            std::string file((strDir+wxFILE_SEP_PATH +filename).mb_str(wxConvUTF8));
            std::cout << file << std::endl;
            Image *img = new Image(file);
            images.push_back(img);
            int index = imageList->Add(wxBitmap(img->getOriginal()));
            ListCtrl->InsertItem(index,index);
        }

        app->setImages(images);
    }
}

/*****************************************************************************/
/* Devolve o conjunto de imagens selecionadas
/*****************************************************************************/
std::vector<Image*> ImagesDialog::getImages() {
    return images;
}

/*****************************************************************************/
/* Limpa o conjunto de imagens selecionadas
/*****************************************************************************/
void ImagesDialog::OnbtnClearClick(wxCommandEvent& event) {
    images.clear();
    ListCtrl->ClearAll();
    imageList->RemoveAll();
}
