#ifndef MUNKRES_H
#define MUNKRES_H

#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/vector.hpp>

using namespace boost::numeric::ublas;

class Munkres {
	public:
		void compute (const matrix<double> &Cost, vector<int> &results);
	private:
		int step;
		matrix<double> Cost;
        std::vector<int> rowCov;
        std::vector<int> colCov;
		matrix<double> M;
		matrix<int> path;
		int Z0_r, Z0_c;
		void step1();
		void step2();
		void step3();
		void step4();
		void step5();
		void step6();
		void find_a_zero(int &row, int &col);
		int star_in_row(int row);
		void find_star_in_row (int row, int *col);
		void find_star_in_col(int c, int *r);
		void find_prime_in_row(int r, int *c);
		void convert_path(int count);
		void clear_covers();
		void erase_primes();
		double find_smallest();
};

#endif

