#include "OriginalShapeContext.h"
#include <iostream>

using namespace std;

//*****************************************************************************/
//* Calcula o shape context
//*****************************************************************************/
void OriginalShapeContext::compute(std::vector<Point> points, double median) {
    int i, j, radBin, angBin, angle;
    double dist;
    int total;

    histogram = matrix<double> (5,12);
        for (i = 0; i < 5; i++)
        for (j = 0; j < 12; j++) {
            histogram(i,j) = 0;
            histogram(i,j) = 0;
        }

	for (unsigned i = 0; i < points.size(); i++) {
        // Invariancia à escala
        dist = hypot(points[i].x - point.x, points[i].y - point.y)/median;
        angle = (int) atan2((point.y - points[i].y),(points[i].x - point.x)) * 180/PI;
        if (dist != 0) {
            if (angle >= 360)
                angle -= 360;
            if (angle < 0)
                angle += 360;
            angBin = angularBin(angle);
            radBin = logRadialBin(dist);
            histogram(radBin,angBin)++;
            histogram(radBin,angBin)++;
        }
	}

    // Normalização
    total = 0;
    for (i = 0; i < 5; i++)
        for (j = 0; j < 12; j++)
            total += histogram(i,j);

    for (i = 0; i < 5; i++) {
        for (j = 0; j < 12; j++) {
            histogram(i,j) = (double)histogram(i,j)/total;
        }
    }

}


/*****************************************************************************/
/* Devolve um bitmap que representa o shape context
/*****************************************************************************/
wxBitmap OriginalShapeContext::toBitmap() {
    int i, j;
    unsigned char value;
	float max = 0;
	int width = 12;
    int height = 5;
    wxImage sc = wxImage(width,height);

	for (i = 0; i < height; i++) {
		for (j = 0; j < width; j++) {
		    if (histogram(i,j) > max)
                max = histogram(i,j);
		}
	}

	for (i = 0; i < height; i++) {
		for (j = 0; j < width; j++) {
            value = 255 - (unsigned char )(histogram(i,j) * 255/max);
            sc.SetRGB(j,i,value,value,value);
        }
	}

	wxImage img;
	img = sc.Scale(width*20, height*20);

    return (wxBitmap(img));
}

/*****************************************************************************/
/* Imprime
/*****************************************************************************/
void OriginalShapeContext::printString() {
    int i, j;
    cout << endl << " Histograma" << endl;
    for (i = 0; i < 5; i++) {
        for (j = 0; j < 12; j++) {
            cout.width(4);
            cout.precision(2);
        }
        cout << endl;
    }
}


//*****************************************************************************/
/* Devolve um bitmap com o diagrama para representacao do shape context
/*****************************************************************************/
wxBitmap OriginalShapeContext::image(int width, int height, std::vector<Point> points, double median) {
    double radius;
    Point pt = getPoint();
    wxBitmap bmp = wxBitmap(width,height);
    wxMemoryDC dc;

    dc.SelectObject(bmp);
    dc.SetPen(wxPen(*wxGREEN, 1, wxSOLID));
    dc.SetBrush(wxBrush(*wxWHITE, wxTRANSPARENT));
    dc.Clear();

    radius = initialRadius * median;
    dc.DrawCircle(pt.x, pt.y, radius);

    for (int i = 1; i < 5; i++) {
        radius = radius * 2;
        dc.DrawCircle(pt.x, pt.y, radius);
    }

    for (int i = 0; i < 12; i++)
        dc.DrawLine(pt.x, pt.y, (pt.x)+radius*cos(i*PI/6),(pt.y)+radius*sin(i*PI/6));

    dc.SetBrush(wxBrush(*wxBLACK, wxSOLID));
    dc.SetPen(wxPen(*wxBLACK, 1, wxSOLID));

    for (unsigned int i = 0; i < points.size(); i++)
        dc.DrawCircle(points[i].x, points[i].y, 2);

    dc.SelectObject(wxNullBitmap);

    return (bmp);
}

/*****************************************************************************/
// Devolve valor armazenado no histograma para o bin dado por (i,j)
/*****************************************************************************/
double OriginalShapeContext::get(int i, int j) {
    return histogram(i,j);
}

/*****************************************************************************/
/* Devolve a distancia em relação a outro shape context                      */
/*****************************************************************************/
double OriginalShapeContext::distance(ShapeContext *sc) {
    double distance = 0.0, g, h;;

    distance = 0.0;
    for (int k = 0; k < 5; k++) {
        for (int l = 0; l < 12; l++) {
            g = this->get(k,l);
            h = ((OriginalShapeContext*)sc)->get(k,l);
            if (g + h > 0)
                distance += ((g-h)*(g-h))/(g+h);
        }
    }

    return distance;

}
