#include "TPS.h"
#include <boost/numeric/ublas/lu.hpp>
#include <boost/numeric/ublas/matrix.hpp>

#include "Utility.h"


long double TPS::tpsBasicFunction (long double x) {
   long double value;
   value = (x == 0) ? 0 : x * x * log(x*x);
   return value;
}

long double TPS::compute (vector<Point>controlPointsP, vector<Point>controlPointsQ, double m) {
    unsigned int i, j;
    matrix<long double> T(controlPointsP.size(),controlPointsP.size());
    matrix<long double> Se(controlPointsP.size(),3);
    matrix<long double> M(controlPointsP.size() + 3, controlPointsP.size() + 3);
    matrix<long double> M_inv(controlPointsP.size() + 3, controlPointsP.size() + 3);
    matrix<long double> H (controlPointsP.size()+3,2);
    matrix<long double> W(controlPointsP.size(),2);
    matrix<long double> aux(controlPointsP.size(),2);
    matrix<long double> E(2,2);
    long double ro, diffX, diffY;
    long double bendingEnergy;
    double lambda = m;

    controlPoints = controlPointsP;

    C =  matrix<long double> (controlPointsP.size()+3,2);

    // Define a matriz Se
    for (i = 0; i < Se.size1(); i++) {
        Se(i,0) = 1;
        Se(i,1) = controlPointsP(i).x;
        Se(i,2) = controlPointsP(i).y;
    }

    // Define a matriz T
    for (i = 0; i < T.size1(); i++) {
        for (j = 0; j < i; j++) {
            diffX = controlPointsP(i).x - controlPointsP(j).x;
            diffY = controlPointsP(i).y - controlPointsP(j).y;
            ro = sqrt(diffX*diffX + diffY*diffY);
            T(i,j) = tpsBasicFunction(ro);
            T(j,i) = T(i,j);
        }
        // Regularizacao
        T(i,i) = lambda;
    }

    // Define a matriz M
    // Matriz T
    for (i = 0; i < T.size1(); i++)
        for (j = 0; j < T.size2(); j++)
            M(i,j) = T(i,j);
    // Matriz Se e sua transposta
    for (i = 0; i < Se.size2(); i++) {
        for (j = 0; j < Se.size1(); j++) {
            M(i + T.size1(),j) = Se(j,i);

        }
    }
    for (i = 0; i < Se.size1(); i++)
        for (j = 0; j < Se.size2(); j++)
            M(i,j + T.size2()) = Se(i,j);
    // 0's
    for (i = T.size1(); i < M.size1(); i++)
        for (j = T.size2(); j < M.size2(); j++)
            M(i,j) = 0;

    // Define a matriz H
    for (i = 0; i < controlPointsQ.size(); i++) {
        H(i,0) = controlPointsQ(i).x;
        H(i,1) = controlPointsQ(i).y;
    }
    for (i = controlPointsQ.size(); i < H.size1(); i++) {
        H(i,0) = 0;
        H(i,1) = 0;
    }

    // Usa a fatoracao LU para inverter M
    permutation_matrix<std::size_t> pm(M.size1());
    if (lu_factorize(M,pm) != 0) {
        std::cout << "Singular" << std::endl;
        exit(1);
    }
    M_inv.assign(identity_matrix<long double>(M.size1()));
    lu_substitute(M, pm, M_inv);

    // Calcula C = M^-1*H
    C = prod(M_inv,H);

    // Define W
    for (i = 0; i < controlPointsP.size(); i++) {
        W(i,0) = C(i,0);
        W(i,1) = C(i,1);
    }

    // Calcula a energia de dobramento
    aux = prod(T,W);
    E = prod(trans(W),aux);
    bendingEnergy = E(0,0) + E(1,1);

    return (bendingEnergy);
}

std::vector<Point> TPS::mapPoints(std::vector<Point> points) {
    unsigned int i, j, n;
    double x, y, f, diffX, diffY, ro;
    std::vector<Point> mappedPoints(points.size());

    n = controlPoints.size();

    for (i = 0; i < points.size(); i++) {
        x  = C(n,0) + C(n+1,0)*points[i].x + C(n+2,0)*points[i].y;
        y  = C(n,1) + C(n+1,1)*points[i].x + C(n+2,1)*points[i].y;
        for (j = 0; j < n; j++) {
            diffX = points[i].x - controlPoints[j].x;
            diffY = points[i].y - controlPoints[j].y;
            ro = sqrt(diffX*diffX + diffY*diffY);
            f = tpsBasicFunction(ro);
            x += C(j,0) * f;
            y += C(j,1) * f;
        }
        mappedPoints[i].x = x;
        mappedPoints[i].y = y;
    }
    return mappedPoints;

}

TPS::TPS()
{
    //ctor
}

TPS::~TPS()
{
    //dtor
}

