#include "Utility.h"
#include <time.h>

bool compareDistanceAndImage (DistanceAndImage a, DistanceAndImage b) { return (a.get<0>() < b.get<0>()); }
bool compareCost (CostAndIndices a, CostAndIndices b) { return (a.get<0>() < b.get<0>()); }
bool compareDistanceAndLetter (DistanceAndLetter a, DistanceAndLetter b) { return (a.get<0>() < b.get<0>()); }
bool comparePair (SCPairLetter a, SCPairLetter b) { return (a.get<0>() < b.get<0>()); }
bool compareAverage (tuple<double,std::string> a, tuple<double,std::string> b) { return (a.get<0>() < b.get<0>()); }

//*****************************************************************************/
//* Calcula a distância entre todos os pares de pontos e devolve a mediana
//*****************************************************************************/
double median (std::vector<Point> points) {
    if (points.size() == 0)
        return 0;

    int i, j;
    double m;
    int num = points.size();
    std::vector<double> distance;
    double dist;
    Point point;

    for (i = 0; i < num; i++) {
        point = points[i];
        for (j = i; j < num; j++) {
            dist = hypot(points[j].x - point.x, points[j].y - point.y);
            distance.push_back(dist);
        }
    }

    sort(distance.begin(), distance.end());
    m = distance[distance.size()/2];
    return (m);
}


//*****************************************************************************/
//* Devolve a matriz de custos
//*****************************************************************************/
matrix<double> computeCost (Image *img1, double median1, Image *img2, double median2) {

    img1->computeShapeContexts(50, median1);
    img2->computeShapeContexts(150, median2);

    std::vector<ShapeContext*> SC1 = img1->getShapeContexts();
    std::vector<ShapeContext*> SC2 = img2->getShapeContexts();


    matrix<double> Cost (SC1.size(),SC2.size());
    for (unsigned int i = 0; i < SC1.size(); i++) {
        ShapeContext *sc1 = SC1[i];
        for (unsigned int j = 0; j < SC2.size(); j++) {
            ShapeContext *sc2 = SC2[j];
            Cost(i,j) = sc1->distance(sc2);
        }
    }

    return Cost;
}

//*****************************************************************************/
//* Calcula a diferenca de tempo
//*****************************************************************************/
double diffclock(clock_t clock1,clock_t clock2) {
	double diffticks=clock1-clock2;
	double diffms=(diffticks*1000)/CLOCKS_PER_SEC;
	return diffms;
}


//*****************************************************************************/
//* Devolve a tangente associada à configuração de pixels dada pela imagem img
//* Deve ser substituido para um metodo mais robusto
//*****************************************************************************/
Tangent tangent(wxImage img) {
    Tangent v;
    v.x = 0;
    v.y = 0;

    bool p11 = (img.GetGreen(0,0) == 0);
    bool p12 = (img.GetGreen(0,1) == 0);
    bool p13 = (img.GetGreen(0,2) == 0);
    bool p21 = (img.GetGreen(1,0) == 0);
    bool p22 = (img.GetGreen(1,1) == 0);
    bool p23 = (img.GetGreen(1,2) == 0);
    bool p31 = (img.GetGreen(2,0) == 0);
    bool p32 = (img.GetGreen(2,1) == 0);
    bool p33 = (img.GetGreen(2,2) == 0);

    //************************************/
    //* 0
    //************************************/
    if (!p11 && !p12 && !p13 && p21 & p22 & p23 & !p31 & !p32 & !p33) {
        v.x = 1; v.y = 0;
    }
    //************************************/
    //* 20
    //************************************/
    else if (!p11 && !p12 && p13 && p21 & p22 & p23 & !p31 & !p32 & !p33) {
        v.x = 0.939692621; v.y = 0.342020143;
    }
    else if (!p11 && p12 && p13 && p21 & p22 & !p23 & !p31 & !p32 & !p33) {
        v.x = 0.939692621; v.y = 0.342020143;
    }
    else if (!p11 && !p12 && !p13 && p21 & p22 & p23 & p31 & !p32 & !p33) {
        v.x = 0.939692621; v.y = 0.342020143;
    }
    else if (!p11 && !p12 && !p13 && !p21 & p22 & p23 & p31 & !p32 & !p33) {
        v.x = 0.939692621; v.y = 0.342020143;
    }
    else if (!p11 && !p12 && !p13 && p21 & p22 & !p23 & !p31 & p32 & p33) {
        v.x = 0.939692621; v.y = 0.342020143;
    }
    else if (!p11 && !p12 && !p13 && p21 & p22 & !p23 & !p31 & !p32 & p33) {
        v.x = 0.939692621; v.y = 0.342020143;
    }
    else if (p11 && !p12 && !p13 && !p21 & p22 & p23 & !p31 & !p32 & p33) {
        v.x = 0.939692621; v.y = 0.342020143;
    }
    //************************************/
    //* 45
    //************************************/
    else if (!p11 && !p12 && p13 && !p21 & p22 & !p23 & p31 & !p32 & !p33) {
        v.x = 0.707106781; v.y = 0.707106781;
    }
    else if (!p11 && p12 && !p13 && p21 & p22 & !p23 & !p31 & !p32 & !p33) {
        v.x = 0.707106781; v.y = 0.707106781;
    }
    else if (!p11 && !p12 && p13 && !p21 & p22 & p23 & p31 & p32 & !p33) {
        v.x = 0.707106781; v.y = 0.707106781;
    }
    else if (!p11 && !p12 && p13 && !p21 & p22 & !p23 & p31 & p32 & !p33) {
        v.x = 0.707106781; v.y = 0.707106781;
    }
    //************************************/
    //* 70
    //************************************/
    else if (!p11 && p12 && p13 && !p21 & p22 & !p23 & p31 & p32 & !p33) {
        v.x = 0.342020143; v.y = 0.939692621;
    }
    else if (!p11 && p12 && !p13 && !p21 & p22 & !p23 & p31 & p32 & !p33) {
        v.x = 0.342020143; v.y = 0.939692621;
    }
    else if (!p11 && !p12 && p13 && !p21 & p22 & p23 & !p31 & p32 & !p33) {
        v.x = 0.342020143; v.y = 0.939692621;
    }
    //************************************/
    //* 90
    //************************************/
    else if (!p11 && p12 && !p13 && !p21 & p22 & !p23 & !p31 & p32 & !p33) {
        v.x = 0; v.y = 1;
    }
    else if (p11 && !p12 && !p13 && p21 & p22 & !p23 & p31 & !p32 & !p33) {
        v.x = 0; v.y = 1;
    }

    else if (!p11 && !p12 && p13 && !p21 & p22 & p23 & !p31 & !p32 & p33) {
        v.x = 0; v.y = 1;
    }
    else if (p11 && !p12 && !p13 && !p21 & p22 & !p23 & p31 & !p32 & !p33) {
        v.x = 0; v.y = 1;
    }
    else if (!p11 && p12 && !p13 && !p21 & p22 & p23 & !p31 & p32 & !p33) {
        v.x = 0; v.y = 1;
    }
    //************************************/
    //* 110
    //************************************/
    else if (!p11 && p12 && !p13 && !p21 & p22 & p23 & !p31 & !p32 & p33) {
        v.x = -0.342020143; v.y = 0.939692621;
    }
    else if (p11 && !p12 && !p13 && p21 & p22 & p23 & !p31 & !p32 & !p33) {
        v.x = -0.342020143; v.y = 0.939692621;
    }
    else if (p11 && !p12 && !p13 && !p21 & p22 & !p23 & !p31 & p32 & !p33) {
        v.x = -0.342020143; v.y = 0.939692621;
    }
    else if (p11 && !p12 && !p13 && p21 & p22 & !p23 & !p31 & p32 & !p33) {
        v.x = -0.342020143; v.y = 0.939692621;
    }
    else if (p11 && !p12 && !p13 && p21 & p22 & !p23 & !p31 & !p32 & p33) {
        v.x = -0.342020143; v.y = 0.939692621;
    }
    //************************************/
    //* 135
    //************************************/
    else if (p11 && !p12 && !p13 && !p21 & p22 & !p23 & !p31 & !p32 & p33) {
        v.x = -0.707106781; v.y = 0.707106781;
    }
    else if (p11 && p12 && !p13 && !p21 & p22 & p23 & !p31 & !p32 & p33) {
        v.x = -0.707106781; v.y = 0.707106781;
    }
    else if (p11 && !p12 && !p13 && p21 & p22 & p23 & !p31 & !p32 & p33) {
        v.x = -0.707106781; v.y = 0.707106781;
    }
    else if (p11 && !p12 && !p13 && p21 & p22 & !p23 & !p31 & p32 & p33) {
        v.x = -0.707106781; v.y = 0.707106781;
    }
    else if (!p11 && !p12 && !p13 && p21 & p22 & !p23 & !p31 & p32 & !p33) {
        v.x = -0.707106781; v.y = 0.707106781;
    }
    else if (p11 && p12 && !p13 && !p21 & p22 & p23 & p31 & !p32 & p33) {
        v.x = -0.707106781; v.y = 0.707106781;
    }
    //************************************/
    //* 160
    //************************************/
    else if (p11 && p12 && !p13 && !p21 & p22 & p23 & !p31 & !p32 & !p33) {
        v.x = -0.939692621; v.y = 0.342020143;
    }
    else if (p11 && !p12 && !p13 && p21 & p22 & p23 & !p31 & !p32 & !p33) {
        v.x = -0.939692621; v.y = 0.342020143;
    }
    else if (!p11 && !p12 && !p13 && p21 & p22 & !p23 & !p31 & p32 & p33) {
        v.x = -0.939692621; v.y = 0.342020143;
    }
    else if (!p11 && !p12 && !p13 && p21 & p22 & !p23 & !p31 & !p32 & p33) {
        v.x = -0.939692621; v.y = 0.342020143;
    }

    return v;
}


//*****************************************************************************/
//* Devolve uma amostra aleatoria de pontos da sequência de pontos fornecida
//*****************************************************************************/
void sample(std::vector<Point> allpts, int num, std::vector<Point> &points) {
    int n, end, aux;
    srand ( time(NULL) );

    std::vector<int> v(allpts.size());
    for(unsigned int i = 0; i < v.size(); i++) {
        v[i] = i;
    }
    end = allpts.size();

    points.clear();
    for (int i = 0; i < num; i++) {
        n = rand()%end;
        points.push_back(allpts[v[n]]);
        aux = v[n];
        v[n] = v[end-1];
        v[end-1] = aux;
        end--;
    }
}
